;***************************************************************************
; MCUEZ12 source code...
;  HELLO.ASM
;	Simple example program for testing the Axiom HC(s)12 boards with paged memory.
;	A text string is sent to the terminal using COM1 / SCI0.
;	works with MON12 user mode or Auto Start
;
;  REv 1: Ported for CML12SDP256 rev F. board, 8Mhz Eclock.
;  Rev 2: Ported older rev C/D board, 4 mhz eclock
;...........................................................................

;**************
;   EQUATES for DP256memory map
;**************
RAMSTRT:	EQU	$1000		; start of internal ram
RAMSIZE:	EQU	$2000		; Size of internal ram, use 8K of 12K
REGBS:  	EQU	$0000		; start of registers
STACK:		EQU	RAMSTRT+RAMSIZE	; top of stack
ROMSTRT		EQU	$4000		; define start of rom space for constants and program
;PRGSTRT:	EQU	PAGE_14		; start of this program running under monitor
; defined after PAGe defintions, no forward reference allowed.

I_VECTOR	EQU	$FF8C		; DP256 physical vector table location
	
;******************************************************************
; Define clock timing based constants...

EEDIV		equ	$15		; ECLKDIV value, 4Mhz osc clock, 190Khz
*FEDIV		equ	$15		; FCLKDIV value, 4Mhz osc clock, 190Khz

* 4Mhz Eclock constants...
BAUD  EQU	$1A     ; 4Mhz Eclock 9600 baud setting for SCI port
;BAUD	EQU	$0d     ; 19,2K baud setting for SCI port
CLKX	EQU	$00	  ; SYNR setting 8MHz


* 8Mhz Eclock constants...
;BAUD  EQU	$34     ; 8Mhz Eclock 9600 baud setting for SCI port
;BAUD	EQU	$1A     ; 19,2K baud setting for SCI port
;CLKX	EQU	$01	  ; SYNR setting 8MHz

;* 16Mhz Eclock constants...
;BAUD  EQU	$68     ; 9600 baud setting for SCI port
;BAUD	EQU	$34     ; 19,2K baud setting for SCI port
;CLKX	EQU	$03	  ; SYNR setting 16MHz

;* 24Mhz Eclock constants...
;BAUD	EQU	$9c     ; 9600 baud setting
;BAUD	EQU	$4E     ; 19,2K baud setting
;CLKX	EQU	$05	  ; SYNR setting for 24MHz

;* define other constants...

EOT:    EQU	$04	; end of text/table character

;*******************************************************************
; define I/O registers...
;*******************************************************************

	include "dp256reg.asm"

;*******************************************************************

; Define MON12 ram vectors if needed, remove if not MON12 application...
; HC12 DP256 table...
	ORG	$3F8A
V_TBL:
VSRV		RMB	2	; RESERVED, VECTOR SERVICE VARIABLE
VPWME		RMB	2	; 8C PWM Emergency shutdown
VPTPI		RMB	2	; 8E Port P input
VC4TX		RMB	2	; 90 CAN4 Transmit
VC4RX		RMB	2	; 92 CAN4 Receive
VC4ERR		RMB	2	; 94 CAN4 Error
VC4WU		RMB	2	; 96 CAN4 wakeup
VC3TX		RMB	2	; 98 CAN3 Transmit
VC3RX		RMB	2	; 9A CAN3 Receive
VC3ERR		RMB	2	; 9C CAN3 Error
VC3WU		RMB	2	; 9E CAN3 wakeup
VC2TX		RMB	2	; A0 CAN2 Transmit
VC2RX		RMB	2	; A2 CAN2 Receive
VC2ERR		RMB	2	; A4 CAN2 Error
VC2WU		RMB	2	; A6 CAN2 wakeup
VC1TX		RMB	2	; A8 CAN1 Transmit
VC1RX		RMB	2	; AA CAN1 Receive
VC1ERR		RMB	2	; AC CAN1 Error
VC1WU		RMB	2	; AE CAN1 wakeup
VC0TX		RMB	2	; B0 CAN0 Transmit
VC0RX		RMB	2	; B2 CAN0 Receive
VC0ERR		RMB	2	; B4 CAN0 Error
VC0WU		RMB	2	; B6 CAN0 wakeup
VFEE		RMB	2	; B8 Flash
VEEP		RMB	2	; BA EEprom
VSPI2		RMB	2	; BC SPI2
VSPI1		RMB	2	; BE SPI1
VI2C		RMB	2	; C0 IIC
VBDLC		RMB	2	; C2 BDLC
VCRGC		RMB	2	; C4 CRG self clock
VCRGL		RMB	2	; C6 CRG Lock
VPABO		RMB	2	; C8 Pulse count B overflow
VMCNT		RMB	2	; CA Modulus counter
VPTHI		RMB	2	; CC Port H input
VPTJI		RMB	2	; CE Port J input
VADC1		RMB	2	; D0 ADC1
VADC0		RMB	2	; D2 ADC0
VSCI1		RMB	2	; D4 SCI1
VSCI0		RMB	2	; D6 SCI0
VSPI0		RMB	2	; D8 SPI0
VPAAI		RMB	2	; DA Pulse count A input
VPAAO		RMB	2	; DC Pulse count A overflow
VTOF		RMB	2	; DE Timer overflow
VTC7		RMB	2	; E0 TC7
VTC6		RMB	2	; E2 TC6
VTC5		RMB	2	; E4 TC5
VTC4		RMB	2	; E6 TC4
VTC3		RMB	2	; E8 TC3
VTC2		RMB	2	; EA TC2
VTC1		RMB	2	; EC TC1
VTC0		RMB	2	; EE TC0
VRTI		RMB	2	; F0 RTI
VIRQ		RMB	2	; F2 IRQ
VXIRQ		RMB	2	; F4 XIRQ
VSWI		RMB	2	; F6 SWI
VTRAP		RMB	2	; F8 TRAP
VCOP		RMB	2	; FA COP
VCLM		RMB	2	; FC Clock
ENDVECT EQU   *

;*************************************************************************
; DEFINE Memory Pages for assembler....

;* Define External Ram memory program pages for program paging Emulation
;	or internal flash pages. 
;* change MEM_BS value to base PPage value for application.
; page label defines linear memory map start address
; page equate value defines PAGE register value or CALL instructon 
;  	page value 

MEM_BS		equ	$20	; define ram pages base address
; MEM_BS		equ	$30	; define flash pages base address

Page_P0_3:	equ	MEM_BS		; ppage register value

Page_P0:	equ	MEM_BS		; ppage register value
Page_P1:	equ	MEM_BS+$01	; ppage 1
Page_P2:	equ	MEM_BS+$02	; ppage 2
Page_P3:	equ	MEM_BS+$03	; ppage 3
Page_P4:	equ	MEM_BS+$04	; ppage 4
Page_P5:	equ	MEM_BS+$05	; ppage 5
Page_P6:	equ	MEM_BS+$06	; ppage 6
Page_P7:	equ	MEM_BS+$07	; ppage 7
Page_P8:	equ	MEM_BS+$08	; ppage 8
Page_P9:	equ	MEM_BS+$09	; ppage 9
Page_PA:	equ	MEM_BS+$0A	; ppage A
Page_PB:	equ	MEM_BS+$0B	; ppage B
Page_PC:	equ	MEM_BS+$0C	; ppage C
Page_PD:	equ	MEM_BS+$0D	; ppage D
Page_PE:	equ	MEM_BS+$0E	; ppage E, Flash mem: this is the Autostart code page also
Page_PF:	equ	MEM_BS+$0F	; ppage F, Flash mem: monitor space if applied

;* define Program Page locations for assembler memory map..
;* Pages defined with extended linear address for MCUez use...
; change PAGE_BS defintion to relocate pages from Ram to flash memory

PAGE_BS:	equ	$80000	; define Ram emulation map base page address
; PAGE_BS:	equ	$C0000	; define Flash map base page address

; define 256K paged memory, 16 x 16K byte memory pages in extended addressing ...
PAGE_0:		equ	PAGE_BS		; define page 0 at lowest page
PAGE_1:		equ	PAGE_BS+$4000	; define page 1
PAGE_2:		equ	PAGE_BS+$8000	; define page 2
PAGE_3:		equ	PAGE_BS+$C000	; define page 3
PAGE_4:		equ	PAGE_BS+$10000	; define page 4
PAGE_5:		equ	PAGE_BS+$14000	; define page 5
PAGE_6:		equ	PAGE_BS+$18000	; define page 6
PAGE_7:		equ	PAGE_BS+$1C000	; define page 7
PAGE_8:		equ	PAGE_BS+$20000	; define page 8
PAGE_9:		equ	PAGE_BS+$24000	; define page 9
PAGE_10:	equ	PAGE_BS+$28000	; define page 10
PAGE_11:	equ	PAGE_BS+$2C000	; define page 11
PAGE_12:	equ	PAGE_BS+$30000	; define page 12
PAGE_13:	equ	PAGE_BS+$34000	; define page 13
;* note: page 14 is also the linear flash memory $4000 > $7FFF
PAGE_14:	equ	PAGE_BS+$38000	; define page 14
;* note: page 15 is also the linear flash memory $C000 > $FFFF
PAGE_15:	equ	PAGE_BS+$3C000	; define page 15


;***************************************************************
; Define ram variables or memory here...

	org	RAMSTRT

TMP	DS	1	; define a variable location

RAMEND	EQU	*	; define ram end location, see where it is

;************************************************************************************
; * Start of ROM area...

	org	ROMSTRT

;***************************************************************
; Program Code location - $4000 - $7FFF 64K map linear space, also PPage $3E,
;	or $F8000 - $FBFFF extended paged space *
;	this location is defined for use under MON12 for testing or Autostart.
;
; For no monitor or BDM use, this should be changed to location:
; 	$C000 - $FFFF 64K map linear space,also PPage $3F,
;	or $FC000 - $FFFFF extended paged space. Note that vectors
;	must also be added at the physical memory location $FF80 > $FFFF
;	for this type of memory map. 
;**************


;****************
; Assembler Macro definition for Program Paging...
; Define absolute mode paging for assembler...
; This macro computes absolute address for CALL instruction then performs
; the CALL instruction with the computed value.

; 1st parameter = Label to goto on new page
; 2nd parameter = Page number desired
; Index Y is applied, contents lost

; Example: CALL_PAGE LABEL,Page_val  
;					Label = routine name on new page
;					Page_val = 

CALL_PAGE	MACRO	
	PSHD			; SAVE ACCUMULATOR
	LDD	#\1.W		; GET GOTO ADDRESS IN PAGE
	anda	#$3F		; MASK OFF NON-PAGE ADDRESS BITS
	oraa	#$80		; SET PAGE ADDRESS BIT
	TFR	D,Y		; PLACE IN ADDRESS POINTER
	puld			; RESTORE ACCUMULATOR
	CALL	0,Y,\2		; GOT THE NEW PAGE AND ROUTINE

	ENDM			
; END OF MACRO CODE SECTION

;.................................................................................

; start code memory section...

; start with machine intialization....
RESET:
	sei			; Interrupts off! in case of trap here
	LDS	#STACK		; stack is set
	CLR     COPCTL		; COP TIMER OFF

; Perform this section if autostart or no monitor operation...
; HC(S)12 mode = Single chip default, change to expanded wide here...

;* enable bus controls and set memory map, COMMENT OUT for Single chip mode...
;*	movw	#$0CE0,PEAR	; NO external Paging mode, EXPANDED WIDE BUS ON, 
;*				; PE4=E, PE3=LSTRB, PE2=R/W
;	movw	#$0CE2,PEAR	; Expanded paging enabled, EXPANDED WIDE BUS ON,
				; PE4=E, PE3=LSTRB, PE2=R/W, PORTK = addresses
;	movb  #$0F,MISC		; Expanded = Low Flash off, ext. mem stretch = 3 
;* end of mode and bus controls set up.

;* set internal eeprom clock for programming access...
	movb	#EEDIV,ECLKDIV	; set EE clock divider

;* enable PLL clock if needed, example is 4Mhz crystal to 8 Mhz bus or E clock...

;	movb	#CLKX,SYNR	; set Eclock frequency
;			* wait for PLL lock...
;PLL_Wait:
;	ldaa	CRGFLG		; get PLL flags
;	bita	#$08		; test for lock
;	beq	PLL_Wait	; wait for lock
;	movb	#$80,CLKSEL	; PLL on...

; Initialize Interrupt vectors....

;	jsr	SET_VECT	; MON12 map: set any interrupt vectors

;................................................................................

; Prepare for Main program on Program Pages and go execute...

	CALL_PAGE START,Page_P0	; GOTO A PROGRAM PAGE TO START MAIN PROGRAM


;* this example starts paging code on 1st. program page at the START label.

;......................................................................................
; Subroutine: set any needed interrupt vectors under MON12

SET_VECT:
	MOVW	#RESET,VCLM		; set clock fail to Reset vector
					; Set all other not applied interrupt vectors
					; to Reset or Monitor may trap as an
					; unscheduled interrupt and application
					; will not recover without user intervention
;	MOVW	#IRQ_SRV,VXIRQ		; Example set xIRQ vector if ISR label = IRQ_SRV
	rts				; return from subroutine
					; Note: IRQ_SRV will be in this section 

;.........................................................................................
; Define global constants and strings where all pages can access...

;**************
; TEXT TABLES *
;**************

MSG:	dc.b   'Hello World, Press any key for message, Press X to quit'
	dc.b   EOT

TABLE_END:	EQU	*


;* End of Monitor map Primary program page $4000 > $7FFF, Flash PPAGE $3E,
;	Flash extended $F8000 > $FBFFF

;*********************************************************************************
;*********************************************************************************
;* Paged program section....
;* user must verfiy that no page exceeds 16K bytes or $4000 hex bytes in length
;	may be possible to do this with an assembler directive...
;*********************************************************************************

	ORG	PAGE_0

;...............................................................................
;
; Main program loop start ....
; 	Starts on 1St. Program Page in memory map

START:
	CALL_PAGE  ONSCI0,Page_P1	; init SCI on different page
MAIN:
	CALL_PAGE  PROMPT,Page_P2	; do the message out a different page
LOOP:
	CALL_PAGE  INSCI0,Page_P3	; check for incomming bytes
	BCC	LOOP			; wait on receive
	cmpa	#'x'			; test if stop command
	beq	MONITOR			; if yes, return to monitor
	cmpa	#'X'			; test for upper case stop command
	bne	MAIN			; if not, send message again
;	BRA	LOOP		; end program here
MONITOR:
	JMP	$C000		; go back to MON12


;**********************************************************************************
; New program page here...

	ORG	PAGE_1

; Initialize the SCI0 for 9600 baud
ONSCI0:
	NOP
	movw	#BAUD,SC0BDH	; set baud rate constant
	ldaa	#$00		; configure SCI0 control registers
	staa	SC0CR1
	ldaa	#$0C		; enable transmit and receive
	staa	SC0CR2
	RTC


;********************************************************************************
; New page here...

	ORG	PAGE_2

PROMPT:
	NOP
	LDX  #MSG	 ; get message string
	JSR  OUTSTRG	 ; send it out serial port
	JSR  OUTCRLF	 ; output carriage-return
	RTC		 ; return from call


; Output string of ASCII bytes starting at x until end of text ($04).
OUTSTRG:
	JSR  OUTCRLF	; output carriage-return
OUTSTRG0:
	PSHA           ; save a
OUTSTRG1:
	LDAA 0,X	; read char into a
        CMPA #EOT      ; is this end of text?
        BEQ  OUTSTRG3	; jump if yes
        JSR  OUTPUT	; output character
        INX            ; incriment pointer
        BRA  OUTSTRG1  ; loop
OUTSTRG3:
	PULA		; restore a
        RTS

; Output a Carriage return.  Returns a = cr.
OUTCRLF:
	LDAA #$0D	; get CR
	JSR  OUTPUT	; send it
	LDAA #$0A	; get LF
	JSR  OUTPUT	; send it
	LDAA #$00
	JSR  OUTPUT	; output padding
	LDAA #$0D
	RTS

; Output A to SCI0
OUTPUT:
OUTSCI2:
	LDAB	SC0SR1	; read status
	BITB	#$80	; test Transmit Data Register Empty bit
	BEQ	OUTSCI2	; loop if TDRE=1
	ANDA	#$7F	; mask parity
	STAA	SC0DRL	; send character
	RTS

;********************************************************************************
; New page here...

; add offset to test macro operation...

	ORG	PAGE_3+1234

**********
*   INSCI() - Read from SCI.  Return a=char or 0.
**********

INSCI0:
	clc			; add flag for recve valid
        LDAA    SC0SR1          ; read status reg
        ANDA    #$20            ; check rdrf
        BEQ     INSCI01         ; jump if no data
        LDAA    SC0DRL          ; read data
	sec			; set flag for data in..
INSCI01:
        RTC


;**************************************************************************
;**************************************************************************

; Define $c000 > $FFFF code and program here, normally reserved for monitor if applied.
; also = PPAGE xF or highest program page.  

;********************************************************************************************
; Include interrupt vectors here for BDM or no monitor, not needed for Monitor application...
; commented out for monitor application

;	ORG    I_VECTOR
;	include "DP256_Ivectors.asm


;* End of Primary program page $C000 > $FFFF, PPAGE $#F, extended $FC000 > $FFFFF


         END

